pub struct SurbBalancer<C, E, F> {
session_id: SessionId,
controller: C,
surb_estimator: E,
flow_control: F,
state: Arc<BalancerStateValues>,
last_estimator_state: SimpleSurbFlowEstimator,
last_update: Instant,
last_decay: Instant,
was_below_target: bool,
}Expand description
Runs a continuous process that attempts to evaluate and regulate the flow of SURBs to the Session counterparty, to keep the number of SURBs locally or at the counterparty at a certain level.
Internally, the Balancer uses an implementation of SurbBalancerController to
control the rate of SURBs consumed or sent to the counterparty
each time the update method is called:
- The size of the SURB buffer at locally or at the counterparty is estimated using
SurbFlowEstimator. - Error against a set-point given in
SurbBalancerConfigis evaluated in theSurbBalancerController. - The
SurbBalancerControllerapplies a new SURB flow rate value using theSurbFlowController.
In the local context, the SurbFlowController might simply regulate the egress traffic from the
Session, slowing it down to avoid fast SURB drainage.
In the remote context, the SurbFlowController might regulate the flow of non-organic SURBs via
Start protocol’s KeepAlive messages to deliver additional
SURBs to the counterparty.
Fields
session_id: SessionIdcontroller: Csurb_estimator: Eflow_control: Fstate: Arc<BalancerStateValues>last_estimator_state: SimpleSurbFlowEstimatorlast_update: Instantlast_decay: Instantwas_below_target: boolImplementations
Sourceimpl<C, E, F> SurbBalancer<C, E, F>where
C: SurbBalancerController + Send + Sync + 'static,
E: SurbFlowEstimator + Send + Sync + 'static,
F: SurbFlowController + Send + Sync + 'static,
impl<C, E, F> SurbBalancer<C, E, F>where
C: SurbBalancerController + Send + Sync + 'static,
E: SurbFlowEstimator + Send + Sync + 'static,
F: SurbFlowController + Send + Sync + 'static,
pub fn new( session_id: SessionId, controller: C, surb_estimator: E, flow_control: F, state: Arc<BalancerStateValues>, ) -> Self
Sourcefn update(&mut self) -> u64
fn update(&mut self) -> u64
Computes the next control update and adjusts the SurbFlowController rate accordingly.
Sourcepub fn start_control_loop(
self,
sampling_interval: Duration,
) -> (impl Stream<Item = u64>, AbortHandle)
pub fn start_control_loop( self, sampling_interval: Duration, ) -> (impl Stream<Item = u64>, AbortHandle)
Spawns a new task that performs updates of the given SurbBalancer at the given sampling_interval.
If cfg_feedback is given, SurbBalancerConfig can be queried for updates and also updated
if the underlying SurbBalancerController also does target updates.
Returns a stream of current estimated buffer levels, and also an AbortHandle
to terminate the loop. If abort_reg was given, the returned AbortHandle corresponds
to it.